function Jacobian = Jacobian_QPHBM(obj,Utot)
% Compute the Jacobian of R with respect to U

H    = obj.H;
nz_tot   = obj.nz_tot;

coef_per_var = (H+1)*4*H+1;

% Extract informations from Utot :
[Ztot,omega1,omega2,lambda,omega1sq,omega2sq,omega1omega2,lambdaomega1,lambdaomega2] = get_Ztot(obj,Utot);

Z_t0 = sum(Ztot(1:(coef_per_var-1)/2+1,:),1);

% definition of the small matrices : I and d
%
% Identity matrix coef_per_var sparse
[iI,jI,vI]=find(speye(coef_per_var)) ;

% derivation indices
cos_ind = 2:(coef_per_var-1)/2+1;
sin_ind = (coef_per_var-1)/2+2:coef_per_var;

Domega1vec = [zeros(H,1) ; reshape((1:H).*ones(2*H+1,1),2*H*H+H,1)];
Domega2vec = [(1:H)' ; reshape((-H:1:H)'.*ones(1,H),2*H*H+H,1)];
Dvec = omega1*Domega1vec+omega2*Domega2vec;
D1vec = lambdaomega1*Domega1vec+lambdaomega2*Domega2vec;
%DLvec = Domega1vec+Domega2vec;
DDvec = omega1sq*(Domega1vec.*Domega1vec) + omega2sq*(Domega2vec.*Domega2vec) + (2*omega1omega2)*(Domega1vec.*Domega2vec);

% d sparse matrix
idmat=[ 1 ,   cos_ind ,  sin_ind ]';
jdmat=[ 1 ,   sin_ind ,  cos_ind ]';
vdmat= [0;Dvec ;-Dvec ];
vd1mat=[0;D1vec;-D1vec];
vdlmat=[0;Dvec;-Dvec];

% dd sparse matrix
iddmat=idmat;
jddmat=iddmat;
vddmat=[0;-DDvec;-DDvec];

% ib and jb for b sparce matrix
[ib,jb,~]=find( sparse( ones(coef_per_var) ));

%   dRdZ =  L0 + lambda L1  + 2*Q(Z,.)
%   construct the list for the large matrix L0, L1, dl, 2*Q(Z,.) and dq(Z,.)
nl0=numel(obj.il0); iL0=zeros(coef_per_var,nl0); jL0=zeros(coef_per_var,nl0); vL0=zeros(coef_per_var,nl0);
for i=1:nl0
    iL0(:,i)=iI + (obj.il0(i)-1)*coef_per_var ;
    jL0(:,i)=jI + (obj.jl0(i)-1)*coef_per_var ;
    vL0(:,i)=vI*obj.vl0(i)  ;
end

nl1=numel(obj.il1); iL1=zeros(coef_per_var,nl1); jL1=zeros(coef_per_var,nl1); vL1=zeros(coef_per_var,nl1);
for i=1:nl1
    iL1(:,i)=iI + (obj.il1(i)-1)*coef_per_var ;
    jL1(:,i)=jI + (obj.jl1(i)-1)*coef_per_var ;
    vL1(:,i)=vI*(obj.vl1(i)*lambda) ;
end

nq=numel(obj.iq); iQL=zeros(coef_per_var^2,nq); jQL=zeros(coef_per_var^2,nq); vQL=zeros(coef_per_var^2,nq);
for i=1:nq
    % Avoid to compute the same B several times / it uses that obj.jq is non-decreasing.
    if ~(i>1 && obj.jq(i-1) == obj.jq(i))
        vb= reshape(obj.B(Ztot(:,obj.jq(i))) ,coef_per_var^2,1) ;
    end
    iQL(:,i)=ib + (obj.iq(i)-1)*coef_per_var ;
    jQL(:,i)=jb + (obj.kq(i)-1)*coef_per_var ;
    vQL(:,i)=2*vb*obj.vq(i) ;
end

nld=numel(obj.id); iD=zeros(coef_per_var,nld); jD=zeros(coef_per_var,nld); vD=zeros(coef_per_var,nld);
for i=1:nld
    iD(:,i)=idmat + (obj.id(i)-1)*coef_per_var ;
    jD(:,i)=jdmat + (obj.jd(i)-1)*coef_per_var ;
    vD(:,i)=vdmat*obj.vd(i) ;
end

nld1=numel(obj.id1); id1=zeros(coef_per_var,nld1); jd1=zeros(coef_per_var,nld1); vd1=zeros(coef_per_var,nld1);
for i=1:nld1
    id1(:,i)=idmat + (obj.id1(i)-1)*coef_per_var ;
    jd1(:,i)=jdmat + (obj.jd1(i)-1)*coef_per_var ;
    vd1(:,i)=vd1mat*obj.vd1(i) ;
end

nldd=numel(obj.idd); idd=zeros(coef_per_var,nldd); jdd=zeros(coef_per_var,nldd); vdd=zeros(coef_per_var,nldd);
for i=1:nldd
    idd(:,i)=iddmat + (obj.idd(i)-1)*coef_per_var ;
    jdd(:,i)=jddmat + (obj.jdd(i)-1)*coef_per_var ;
    vdd(:,i)=vddmat*obj.vdd(i) ;
end



% Assumption : the variable after kdq(i) is the derivative of the variable kdq(i)

nldl=numel(obj.idl); idl=zeros(coef_per_var,nldl); jdl=zeros(coef_per_var,nldl); vdl=zeros(coef_per_var,nldl);
for i=1:nldl
    idl(:,i)=idmat + (obj.idl(i)-1)*coef_per_var ;
    jdl(:,i)=jdmat + (obj.jdl(i)-1)*coef_per_var ;
    vdl(:,i)=vdlmat*obj.vdl(i) ;
end

% Assumption : the variable after kdq(i) is the derivative of the variable kdq(i)
% Assumption : the variable after kdq(i) is the derivative of the variable kdq(i)

ndq=numel(obj.idq); idq1=zeros(coef_per_var^2,ndq); jdq1=zeros(coef_per_var^2,ndq); vdq1=zeros(coef_per_var^2,ndq);
for i=1:ndq
    vb= reshape(obj.B(Ztot(:,obj.jdq(i))),coef_per_var^2,1) ;
    idq1(:,i)=ib + (obj.idq(i)-1)*coef_per_var ;
    % Assumption : the variable after kdq(i) is the derivative of the variable kdq(i)
    jdq1(:,i)=jb + (obj.kdq(i)+1-1)*coef_per_var ;
    vdq1(:,i)=vb*obj.vdq(i) ;
end

idq2=zeros(coef_per_var^2,ndq); jdq2=zeros(coef_per_var^2,ndq); vdq2=zeros(coef_per_var^2,ndq);
for i=1:ndq
    % Assumption : the variable after kdq(i) is the derivative of the variable kdq(i)
    vb= reshape(obj.B(Ztot(:,obj.kdq(i)+1)),coef_per_var^2,1) ;
    idq2(:,i)=ib + (obj.idq(i)-1)*coef_per_var ;
    jdq2(:,i)=jb + (obj.jdq(i)-1)*coef_per_var ;
    vdq2(:,i)=vb*obj.vdq(i) ;
end



idRdZ=[ reshape(iL0,nl0*coef_per_var,1) ; reshape(iL1,nl1*coef_per_var,1) ; ...
    reshape(iQL,nq*coef_per_var^2,1) ; reshape(iD,nld*coef_per_var,1); ...
    reshape(id1,nld1*coef_per_var,1) ; reshape(idd,nldd*coef_per_var,1); ...
    reshape(idq2,ndq*coef_per_var^2,1) ; reshape(idq1,ndq*coef_per_var^2,1) ; reshape(idl,nldl*(coef_per_var),1)];

jdRdZ=[ reshape(jL0,nl0*coef_per_var,1) ; reshape(jL1,nl1*coef_per_var,1) ; ...
    reshape(jQL,nq*coef_per_var^2,1) ; reshape(jD,nld*coef_per_var,1); ...
    reshape(jd1,nld1*coef_per_var,1) ; reshape(jdd,nldd*coef_per_var,1); ...
    reshape(jdq2,ndq*coef_per_var^2,1) ; reshape(jdq1,ndq*coef_per_var^2,1) ; reshape(jdl,nldl*(coef_per_var),1)];

vdRdZ=[ reshape(vL0,nl0*coef_per_var,1) ; reshape(vL1,nl1*coef_per_var,1) ; ...
    reshape(vQL,nq*coef_per_var^2,1) ; reshape(vD,nld*coef_per_var,1); ...
    reshape(vd1,nld1*coef_per_var,1) ; reshape(vdd,nldd*coef_per_var,1); ...
    reshape(vdq2,ndq*coef_per_var^2,1) ; reshape(vdq1,ndq*coef_per_var^2,1) ; reshape(vdl,nldl*(coef_per_var),1)];

dRdZ=sparse(idRdZ,jdRdZ,vdRdZ,coef_per_var*nz_tot,coef_per_var*nz_tot) ;

%   Modification for dl/dq : initial condition replace the zero-th harmonic :
dRdZ(coef_per_var*(obj.idl-1)+1,:) = 0;
for i=1:numel(obj.idl)
    dRdZ(coef_per_var*(obj.idl(i)-1)+1,coef_per_var*(obj.jdl(i)-1)+(1:(coef_per_var+1)/2)) = obj.vdl(i);
end
for i=1:numel(obj.idq)
    dRdZ(coef_per_var*(obj.idq(i)-1)+1,coef_per_var*(obj.kdq(i)-1)+(1:(coef_per_var+1)/2)) = obj.vdq(i)*Z_t0(obj.jdq(i))  +  dRdZ(coef_per_var*(obj.idq(i)-1)+1,coef_per_var*(obj.kdq(i)-1)+(1:(coef_per_var+1)/2));
end

%   dRd1ambda  = C1 + L1(Z) + 2*lambda*C2
dRdlambda =zeros(coef_per_var,nz_tot);
for i=1:numel(obj.iforce1)
    dRdlambda( (sys.hforce1(i)-1)*(2*H+1)+1,obj.iforce1(i)) = dRdlambda( (sys.hforce1(i)-1)*(2*H+1)+1,obj.iforce1(i)) - obj.vforce1(i);
end
for i=1:numel(obj.iforce2)
    dRdlambda( (sys.hforce2(i)-1)*(2*H+1)+1,obj.iforce2(i)) = dRdlambda( (sys.hforce2(i)-1)*(2*H+1)+1,obj.iforce2(i)) - 2*lambda*obj.vforce2(i);
end
for i=1:numel(obj.ic1)
    dRdlambda(1,obj.ic1(i))= dRdlambda(1,obj.ic1(i)) + obj.vc1(i) ;
end
for i=1:numel(obj.ic2)
    dRdlambda(1,obj.ic2(i))= dRdlambda(1,obj.ic2(i)) + obj.vc2(i)*2*lambda ;
end
for i=1:numel(obj.il1)
    dRdlambda(:,obj.il1(i))= dRdlambda(:,obj.il1(i)) + obj.vl1(i)*Ztot(:,obj.jl1(i)) ;
end
dRdlambda=reshape(dRdlambda,coef_per_var*nz_tot,1);




%   dRdomega1
dRdomega1=zeros(coef_per_var,nz_tot);
for i=1:nld
    dRdomega1(:,obj.id(i))= dRdomega1(:,obj.id(i)) + obj.vd(i)*obj.D(Ztot(:,obj.jd(i)),1,0);
end
for i=1:nldl
    dRdomega1(:,obj.idl(i))= dRdomega1(:,obj.idl(i)) + obj.vdl(i)*obj.D(Ztot(:,obj.jdl(i)),1,0);
end
dRdomega1=reshape(dRdomega1,coef_per_var*nz_tot,1);

%   dRdomega2
dRdomega2=zeros(coef_per_var,nz_tot);
for i=1:nld
    dRdomega2(:,obj.id(i))= dRdomega2(:,obj.id(i)) + obj.vd(i)*obj.D(Ztot(:,obj.jd(i)),0,1) ;
end
for i=1:nldl
    dRdomega2(:,obj.idl(i))= dRdomega2(:,obj.idl(i)) + obj.vdl(i)*obj.D(Ztot(:,obj.jdl(i)),0,1);
end
dRdomega2=reshape(dRdomega2,coef_per_var*nz_tot,1);




%   dRdomega1sq
dRdomega1sq =zeros(coef_per_var,nz_tot);
for i=1:nldd
    dRdomega1sq(:,obj.idd(i))= dRdomega1sq(:,obj.idd(i)) + obj.vdd(i)*obj.DD(Ztot(:,obj.jdd(i)),1,0,0) ;
end
dRdomega1sq=reshape(dRdomega1sq,coef_per_var*nz_tot,1);

%   dRdomega2sq
dRdomega2sq =zeros(coef_per_var,nz_tot);
for i=1:nldd
    dRdomega2sq(:,obj.idd(i))= dRdomega2sq(:,obj.idd(i)) + obj.vdd(i)*obj.DD(Ztot(:,obj.jdd(i)),0,1,0) ;
end
dRdomega2sq=reshape(dRdomega2sq,coef_per_var*nz_tot,1);

%   dRdomega1omega2
dRdomega1omega2 =zeros(coef_per_var,nz_tot);
for i=1:nldd
    dRdomega1omega2(:,obj.idd(i))= dRdomega1omega2(:,obj.idd(i)) + obj.vdd(i)*obj.DD(Ztot(:,obj.jdd(i)),0,0,1) ;
end
dRdomega1omega2=reshape(dRdomega1omega2,coef_per_var*nz_tot,1);

%   dRdlambdaomega1
dRdlambdaomega1 =zeros(coef_per_var,nz_tot);
for i=1:nld1
    dRdlambdaomega1(:,obj.id1(i))= dRdlambdaomega1(:,obj.id1(i)) + obj.vd1(i)*obj.D(Ztot(:,obj.jd1(i)),1,0) ;
end
dRdlambdaomega1=reshape(dRdlambdaomega1,coef_per_var*nz_tot,1);

%   dRdlambdaomega2
dRdlambdaomega2 =zeros(coef_per_var,nz_tot);
for i=1:nld1
    dRdlambdaomega2(:,obj.id1(i))= dRdlambdaomega2(:,obj.id1(i)) + obj.vd1(i)*obj.D(Ztot(:,obj.jd1(i)),0,1) ;
end
dRdlambdaomega2=reshape(dRdlambdaomega2,coef_per_var*nz_tot,1);

% Equation defining omega1sq,omega2sq,omega1omega2,lambdaomega1-2
rowomega1sq = sparse([1 1],[obj.neq-1 obj.ninc-4],[-2*omega1 1],1,obj.ninc);
rowomega2sq = sparse([1 1],[obj.neq obj.ninc-3],[-2*omega2 1],1,obj.ninc);
rowomega1omega2 = sparse([1 1 1],[obj.neq-1 obj.neq obj.ninc-2],[-omega2 -omega1 1],1,obj.ninc);
rowlambdaomega1 = sparse([1 1 1],[obj.neq-1 obj.neq+1 obj.ninc-1],[-lambda -omega1 1],1,obj.ninc);
rowlambdaomega2 = sparse([1 1 1],[obj.neq obj.neq+1 obj.ninc],[-lambda -omega2 1],1,obj.ninc);


% phase equation zi'(0)=0 -> zis1 + 2*zis2 + 3*zis3 + ...  =0
switch obj.subtype
    case {'autonomous','conservative'}
        if obj.zi_phase1 <= obj.nz; indice=(obj.zi_phase1-1)*coef_per_var; else, indice=(obj.zi_phase1-1)*coef_per_var+3; end
        rowphase1=sparse(1,indice + (coef_per_var-1)/2+1+(1+2*H),1,1,obj.ninc) ;
    case 'forced'
        if isfloat(obj.angfreq) % omega1 fixed
            rowphase1=sparse(1,obj.neq-1,1,1,obj.ninc);
        else % omega1 is the continuation parameter
            rowphase1=sparse([1 1],[obj.neq-1 obj.neq+1],[1 -1],1,obj.ninc);
        end
end
if obj.zi_phase2 <= obj.nz; indice=(obj.zi_phase2-1)*coef_per_var; else, indice=(obj.zi_phase2-1)*coef_per_var+3; end
 rowphase2=sparse(1,indice + (coef_per_var-1)/2+2,1,1,obj.ninc) ;

% Block-decomposition of the Jacobian matrix for the condensation of the
% auxiliar variables

Jacobian.dRdU = [dRdZ(1:obj.neq-2,1:obj.neq-2),sparse(dRdomega1(1:obj.neq-2)),sparse(dRdomega2(1:obj.neq-2)),sparse(dRdlambda(1:obj.neq-2)) ; ...
    rowphase1(1:obj.neq+1) ; rowphase2(1:obj.neq+1)];

Jacobian.dRdUaux = [dRdZ(1:obj.neq-2,obj.neq-1:end), sparse(dRdomega1sq(1:obj.neq-2)) , sparse(dRdomega2sq(1:obj.neq-2)) , ... 
    sparse(dRdomega1omega2(1:obj.neq-2)), sparse(dRdlambdaomega1(1:obj.neq-2)) , sparse(dRdlambdaomega2(1:obj.neq-2)) ; ...
    rowphase1(obj.neq+2:end) ; rowphase2(obj.neq+2:end)];

Jacobian.dRauxdU = [dRdZ(obj.neq-1:end,1:obj.neq-2), sparse(dRdomega1(obj.neq-1:end)) , sparse(dRdomega2(obj.neq-1:end)) , sparse(dRdlambda(obj.neq-1:end)) ; ...
    rowomega1sq(1:obj.neq+1); rowomega2sq(1:obj.neq+1); rowomega1omega2(1:obj.neq+1); rowlambdaomega1(1:obj.neq+1); rowlambdaomega2(1:obj.neq+1)];

Jacobian.dRauxdUaux = [dRdZ(obj.neq-1:end,obj.neq-1:end), sparse(dRdomega1sq(obj.neq-1:end)), sparse(dRdomega2sq(obj.neq-1:end)) , ... 
    sparse(dRdomega1omega2(obj.neq-1:end)), sparse(dRdlambdaomega1(obj.neq-1:end)), sparse(dRdlambdaomega2(obj.neq-1:end)) ; ...
    rowomega1sq(obj.neq+2:end) ;rowomega2sq(obj.neq+2:end) ;rowomega1omega2(obj.neq+2:end) ; rowlambdaomega1(obj.neq+2:end); rowlambdaomega2(obj.neq+2:end)];

Jacobian.dRtotdUtot = [[Jacobian.dRdU ; Jacobian.dRauxdU], [Jacobian.dRdUaux ; Jacobian.dRauxdUaux]];

end


