% function Hand = plotbranchHBM(sys,Section,Idisp,Hdisp,bifpara_str)
%    plots the amplitude of the harmonics Hdisp of the variables Idisp computed
%    by the Manlab/HBM method as a function of the bifurcation parameter
%    specified by bifpara_str ( = 'omega' or 'lambda').
%
%    It also displays the bifurcation points:
%    - B for a simple bifurcation (saddle-node, pitchfork...)
%    - PD for period doubling bifurcation
%    - NS for Neimark-Sacker bifurcation
%
%
%    By L. Guillot and O. Thomas / Nov. 2018 - Mars 2019


function Hand = plotdiagHBMbif(sys,Diag,Idisp,Hdisp,bifpara_str)

if ~isstruct(Diag)
    [Diag] = calcdiagUpp(sys,Diag);
end

DiagUpp = Diag.DiagUpp;
Stabinfo = Diag.Stabinfo;
change = Diag.change;

Nb=length(Idisp);
Hand=[];
Handleg=[];

% Color for successive plots.
Col = dispcolorML;

switch bifpara_str
    case 'omega'
        ind_bifpara = sys.neq;
    case 'lambda'
        ind_bifpara = sys.neq+1;
end


bifpara = DiagUpp(ind_bifpara,:);

strstab = '-';
strunstab = ':';

H = sys.H;
DHp1 = 2*H+1;

for ii=1:Nb
    idisp = Idisp(ii);
    hdisp = Hdisp(ii);
    
    cdisp = Col(ii,:);
    
    if idisp <= sys.nz
        if hdisp == 0
            I0 = (idisp-1)*DHp1+1;
            Norm = sqrt(DiagUpp(I0,:).^2);
        else
            I0 = 0;
            Icos = (idisp-1)*DHp1+1+hdisp;
            Isin = (idisp-1)*DHp1+1+H+hdisp;
            Norm = sqrt(DiagUpp(Icos,:).^2+DiagUpp(Isin,:).^2);
        end
    else
        if hdisp == 0
            I0 = sys.neq+1+ (idisp-1-sys.nz)*DHp1+1;
            Norm = sqrt(DiagUpp(I0,:).^2);
        else
            I0 = 0;
            Icos = sys.neq+1+ (idisp-1-sys.nz)*DHp1+1+hdisp;
            Isin = sys.neq+1+ (idisp-1-sys.nz)*DHp1+1+H+hdisp;
            Norm = sqrt(DiagUpp(Icos,:).^2+DiagUpp(Isin,:).^2);
        end
    end
        
    if numel(change) == 0
        
        if Stabinfo(1) == 1; strplot = strstab; else; strplot = strunstab; end
        hand = plot(bifpara,Norm,strplot,'Color',cdisp);
        Hand = [Hand;hand];
        
    else
        
        for istab=1:numel(change)
            ind_change = change(istab);
            if istab == 1; indplot = 1:ind_change; else; indplot = change(istab-1):ind_change; end
            if Stabinfo(indplot(2)) == 1; strplot = strstab; else; strplot = strunstab; end
            
            if I0 == 0
                Normstab   = sqrt(DiagUpp(Icos,ind_change).^2+DiagUpp(Isin,ind_change).^2);
            else
                Normstab = sqrt(DiagUpp(I0,ind_change).^2);
            end
            
            bifparastab = DiagUpp(ind_bifpara,ind_change);
            
            hand = plot(bifpara(indplot),Norm(indplot),strplot,'Color',cdisp);
            Hand = [Hand;hand];
            hand = plot(bifpara(ind_change),Norm(ind_change),'.','Color',cdisp);
            Hand = [Hand;hand];
            
            text(bifparastab,Normstab,char(Stabinfo(ind_change)),'margin',3,'verticalalignment','Bottom')
            box on;
        end
        
        if Stabinfo(ind_change+1) == 1; strplot = strstab; else; strplot = strunstab; end
        hand = plot(bifpara(ind_change:end),Norm(ind_change:end),strplot,'Color',cdisp);
        Hand = [Hand;hand];
        
    end
    
    Handleg(ii) = plot(NaN,NaN,'-','Color',cdisp);
    Leg{ii}=['z_',num2str(idisp),' H_',num2str(hdisp)];
end

legend(Handleg,Leg,'location','best'); % Slows the plotting procedure

ylabel('harmonic amplitude')
xlabel(['\' bifpara_str])


end