function [EigenVal,FloqMult] = calcdiagStab_parallel(sys,Diagram)
% function [EigenVal] = calcdiagStab(sys,Diagram)
%
% From the Diagram structure, computes all the Eigen information at all the
% points of discretization of the series and puts them in EigenVal matrix.
%

nb_Section = length(Diagram);
nb_pts = size(Diagram{1}.Upp,2);
nb_pts_diag = nb_Section*nb_pts;

EigenVal = cell(nb_pts,nb_pts_diag);
FloqMult = EigenVal;


parfor isec = 1:nb_Section
    Secii = Diagram{isec};
    
    for ipts = 1:nb_pts

        Uii = Secii.Upp(:,ipts);
        
        eigval = get_eigval(sys,Uii);
        
        %%% Put the eigenvalue in the full matrix.
        nb_eigval = numel(eigval);
        EigenVal{isec,ipts} = eigval;
        FloqMult{isec,ipts} = exp(2*pi/Uii(sys.neq)*eigval);
        if nb_eigval < sys.nz-1
            EigenVal{isec,ipts} = NaN;
            FloqMult{isec,ipts} = NaN;
        end
    
    end

end

EigenVal = EigenVal.';
FloqMult = FloqMult.';
EigenVal = EigenVal(:).';
FloqMult = FloqMult(:).';
bool_mat = ~cellfun('isempty',EigenVal);
%%% Seems not to work... ?!
EigenVal = cell2mat(EigenVal(bool_mat));
FloqMult = cell2mat(FloqMult(bool_mat));

end


function eigval = get_eigval(sys,U)
%%% Computation of the Jacobian matrix at point Uii
Jacobian = sys.Jacobian(U);
%%% Condensation of the auxiliary variables
[Jacobian.LK_aux,Jacobian.UK_aux,Jacobian.pK_aux,Jacobian.qK_aux] = lu(Jacobian.dRauxdUaux,'vector');
Jacobian.Kfull = Jacobian.dRdU - ...
    Jacobian.dRdUaux(:,Jacobian.qK_aux)*(Jacobian.UK_aux\(Jacobian.LK_aux\Jacobian.dRauxdU(Jacobian.pK_aux,:)));
%%% Remove the column dR/dlambda
Jacobian.K = Jacobian.Kfull(:,1:end-1);

%%% Computation of the eigenvalues
eigval = sys.Stability(U,Jacobian);
eigval = sort(eigval);
end